<?php

namespace Drupal\block_condition_published\Plugin\Condition\Deriver;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\Context\EntityContextDefinition;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Deriver that creates a condition for each entity type with publishing.
 */
class PublishedStateDeriver extends DeriverBase implements ContainerDeriverInterface {

  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new EntityBundle.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    $entity_context_definitions = [];
    // Get all entity types.
    foreach ($this->entityTypeManager->getDefinitions() as $entity_type_id => $entity_type) {
      if (!$entity_type->hasKey('published')) {
        continue;
      }

      // Create a context definition object for the 'entity'. This is common
      // to all the derivatives.
      $entity_context_definitions[$entity_type_id] = EntityContextDefinition::create("entity:$entity_type_id")
        ->setRequired(FALSE);
    }

    // Add the full definition to the derivatives array.
    $this->derivatives['entity_published'] = ['context_definitions' => $entity_context_definitions] + $base_plugin_definition;

    return $this->derivatives;
  }

}
